from abc import ABC, abstractmethod


class LedPanelAbstract(ABC):
    """ ########################################################### Basic ####################################################### """

    @abstractmethod
    def connect(self):
        pass

    @abstractmethod
    def disconnect(self):
        pass
    
    @abstractmethod
    def set_program(self, program_number: int):
        pass
    
    @abstractmethod
    def get_serial_number(self):
        pass
    
    @abstractmethod
    def get_sw_version(self):
        pass
    
    @abstractmethod
    def set_program_and_led_type(self, program_number: int, led_type: int):
        pass
    
    """ ##################################################### Parameter setting ################################################# """
    
    @abstractmethod
    def set_startup_program(self, program_number: int):
        pass
    
    @abstractmethod
    def set_program_name(self, program_name: str):
        pass
    
    @abstractmethod
    def set_led_type(self, led_type: int):
        pass
    
    @abstractmethod
    def set_led_output(self, output_parameter: int):
        pass
    
    @abstractmethod
    def confirm_led_output(self):
        pass
    
    """ ########################################################### Autocal ##################################################### """
    
    @abstractmethod
    def set_autocal(self):
        pass
    
    @abstractmethod
    def set_target_luminance(self, luminance: int):
        pass



import unittest
from unittest.mock import patch, MagicMock
from opsys_led_panel.led_panel_controller import LedPanelController


class Test(unittest.TestCase):
    @ classmethod
    def setUp(self):
        pass

    @ classmethod
    def setUpClass(cls):
        pass

    @ classmethod
    def tearDownClass(cls):
        pass

    @ patch.object(LedPanelController, 'connect')
    def test_connect(self, relay_mock: MagicMock):
        port = 'COM3'
        relay_conn = LedPanelController()
        relay_conn.connect(port)
        relay_mock.assert_called_once_with('COM3')

    @ patch.object(LedPanelController, 'disconnect')
    def test_disconnect(self, relay_mock: MagicMock):
        relay_conn = LedPanelController()
        relay_conn.disconnect()
        relay_mock.assert_called_once_with()

    @ patch.object(LedPanelController, 'set_program')
    def test_set_program(self, relay_mock: MagicMock):
        program_number = 1
        relay_conn = LedPanelController()
        relay_conn.set_program(program_number=program_number)
        relay_mock.assert_called_once_with(program_number=1)

    @ patch.object(LedPanelController, 'get_serial_number')
    def test_get_serial_number(self, relay_mock: MagicMock):
        relay_conn = LedPanelController()
        relay_conn.get_serial_number()
        relay_mock.assert_called_once_with()

    @ patch.object(LedPanelController, 'get_sw_version')
    def test_get_sw_version(self, relay_mock: MagicMock):
        relay_conn = LedPanelController()
        relay_conn.get_sw_version()
        relay_mock.assert_called_once_with()

    @ patch.object(LedPanelController, 'set_program_and_led_type')
    def test_set_program_and_led_type(self, relay_mock: MagicMock):
        program_number, led_type = 1, 1
        relay_conn = LedPanelController()
        relay_conn.set_program_and_led_type(program_number, led_type)
        relay_mock.assert_called_once_with(1, 1)
        
    @ patch.object(LedPanelController, 'set_startup_program')
    def test_set_startup_program(self, relay_mock: MagicMock):
        program_number = 1
        relay_conn = LedPanelController()
        relay_conn.set_startup_program(program_number=program_number)
        relay_mock.assert_called_once_with(program_number=1)
        
    @ patch.object(LedPanelController, 'set_program_name')
    def test_set_program_name(self, relay_mock: MagicMock):
        program_name = 'program1'
        relay_conn = LedPanelController()
        relay_conn.set_program_name(program_name=program_name)
        relay_mock.assert_called_once_with(program_name='program1')
        
    @ patch.object(LedPanelController, 'set_led_type')
    def test_set_led_type(self, relay_mock: MagicMock):
        led_type = 1
        relay_conn = LedPanelController()
        relay_conn.set_led_type(led_type=led_type)
        relay_mock.assert_called_once_with(led_type=1)
        
    @ patch.object(LedPanelController, 'set_led_output')
    def test_set_led_output(self, relay_mock: MagicMock):
        output_parameter = 0
        relay_conn = LedPanelController()
        relay_conn.set_led_output(output_parameter=output_parameter)
        relay_mock.assert_called_once_with(output_parameter=0)
        
    @ patch.object(LedPanelController, 'confirm_led_output')
    def test_confirm_led_output(self, relay_mock: MagicMock):
        relay_conn = LedPanelController()
        relay_conn.confirm_led_output()
        relay_mock.assert_called_once_with()
        
    @ patch.object(LedPanelController, 'set_autocal')
    def test_set_autocal(self, relay_mock: MagicMock):
        relay_conn = LedPanelController()
        relay_conn.set_autocal()
        relay_mock.assert_called_once_with()
        
    @ patch.object(LedPanelController, 'set_target_luminance')
    def test_set_target_luminance(self, relay_mock: MagicMock):
        luminance = 20
        relay_conn = LedPanelController()
        relay_conn.set_target_luminance(luminance=luminance)
        relay_mock.assert_called_once_with(luminance=20)
        


if __name__ == '__main__':
    unittest.main()



import time
import serial
from serial import SerialException
from .led_panel_abstract import LedPanelAbstract

# avoild COMs detection at deployment
try:
    from serial.tools.list_ports import comports
except Exception as e:
    print(e)


class LedPanelController(LedPanelAbstract):
    """
    USB LED Panel controller
    """
    __buffer_delimiter = '\r'

    def __init__(self, conn_timeout=15, delay_time = 0.3, baudrate = 9600):
        """
        Initialize parameters

        Args:
            conn_timeout (int, optional): serial connection timeout. Defaults to 15.
            delay_time (float, optional): serial connection delay in sec. Defaults to 0.3.
            baudrate (int, optional): COM port baudrate. Defaults to 9600.
        """
        self.conn_timeout = conn_timeout
        self.delay_time = 0.3
        self.baudrate = 9600
        
    def read_buffer(self):
        """
        Read buffer output

        Returns:
            list: buffer response bytes
        """
        output = b""  # init binary string
        start_time = time.time()

        while True:  # read from buffer
            output += self.conn.read()

            if r'\r' in str(output):
                break
            # stop on timeout
            if (time.time() - start_time) >= self.conn_timeout:
                raise SerialException('Connection timeout!')

        return output.decode('ascii').rstrip('\r').split(',')
    
    def get_cmd_response(self):
        """
        Read sent command response

        Returns:
            str: command response
        """
        response = self.read_buffer()
        self.conn.flush()  # flush output
        
        if response[0] == 'ER1':
            print(f'Error reading response from buffer!')
            return response[0]

        elif response[0] == 'OK':
            return response[1:]
    
    def send_command(self, command):
        """
        Send command to device

        Args:
            command (str): command to send
                           to device
                           
        Returns:
            str: command response
        """
        command += self.__buffer_delimiter
        
        try:
            self.conn.write(command.encode('ascii'))
            return self.get_cmd_response()
        except:
            print(f'Failed to send command: {command}')
            return 'Error'

    def connect(self, com_port=None):
        """
        Connect to led panel

        Args:
            com_port (str, optional): Serial port name.
                                      Defaults to None.
                   
        Returns:
            None
        """
        if com_port is not None:
            self.conn = serial.Serial(com_port, self.baudrate)
            self.conn.timeout = self.delay_time
            return
        else:  # no com port provided
            for com, _, _ in sorted(comports()):
                self.conn = serial.Serial(com, self.baudrate)
                self.conn.timeout = self.delay_time
                # try to receive valid data from buffer
                if self.get_serial_number() != 'Error':
                    return

        raise SerialException("No valid COM port found")

    def disconnect(self):
        """
        Disconnect from LED panel
        """
        self.conn.close()
        
    def set_program(self, program_number=1):
        """
        Set program to LED panel

        Args:
            program_number (int, optional): program number.
                                            Defaults to 1. 
        """
        self.send_command(command=f'P,{program_number}')
        print(f'Set program to P{program_number} done')
        
    def get_serial_number(self):
        """
        Get device serial number
        
        Returns:
            str: device SN
        """
        response = self.send_command(command='RSNO')
        return response[0] if response != 'Error' else 'Error'
    
    def get_sw_version(self):
        """
        Get device SW version
        """
        response = self.send_command(command='VER')
        return response[0] if response != 'Error' else 'Error'
    
    def get_model(self):
        """
        Get device model
        """
        response = self.send_command(command='VER')
        return response[1] if response != 'Error' else 'Error'
    
    def set_program_and_led_type(self, program_number=1, led_type=1):
        """
        Set program and LED type

        Args:
            program_number (int, optional): program number.
                                            Defaults to 1.
            led_type (int, optional): LED type.
                                      Defaults to 1.
        """
        self.send_command(command=f'PL,{program_number},{led_type}')
        print(f'Set startup program to P{program_number} and LED type to {led_type} done')
    
    def set_startup_program(self, program_number=1):
        """
        Set startup program

        Args:
            program_number (int, optional): program number.
                                            Defaults to 1.
        """
        self.send_command(command=f'SPG,{program_number}')
        print(f'Set startup program to P{program_number} done')
    
    def set_program_name(self, program_name):
        """
        Set program name

        Args:
            program_name (str): program name
        """
        self.send_command(command=f'SNAME,{program_name}')
        print(f'Set program name to {program_name} done')
    
    def set_led_type(self, led_type=1):
        """
        Set LED type

        Args:
            led_type (int, optional): LED type.
                                      Defaults to 1.
        """
        self.send_command(command=f'SLT,{led_type}')
        print(f'Set LED type to {led_type} done')
    
    def set_led_output(self, output_parameter):
        """
        Set LED output parameter

        Args:
            output_parameter (int): 0~4095
        """
        self.send_command(command=f'SV,{output_parameter}')
        print(f'Confirm LED output to {output_parameter} done')
        
    def confirm_led_output(self):
        """
        LED output parameter confirmation
        """
        self.send_command(command='RV')
        print('Confirm LED output done')
    
    def set_autocal(self):
        """
        Set AUTOCAL
        """
        self.send_command(command='AC')
        print('Set AUTOCAL done')
    
    def set_target_luminance(self, luminance):
        """
        Set target luminance in AUTOCAL execution

        Args:
            luminance (int): luminance value.
                             Accepted in range 0.0000~30000(cd/m2).
        """
        self.send_command(command=f'SBV,{luminance}')
        print(f'Set target luminance to {luminance} done')



import setuptools
from pathlib import Path


def get_install_requirements():
    """
    Extract packages from requirements.txt file to list

    Returns:
        list: list of packages
    """
    fname = Path(__file__).parent / 'requirements.txt'
    targets = []
    if fname.exists():
        with open(fname, 'r') as f:
            targets = f.read().splitlines()
    return targets

setuptools.setup(name='opsys-led-panel',
                 version='0.0.3',
                 description='python package for led panel device control',
                 url='https://bitbucket.org/opsys_tech/opsys-led-panel/src/master/',
                 download_url='https://bitbucket.org/opsys_tech/opsys-led-panel/src/master/',
                 author='dmitry.borovensky',
                 install_requires=get_install_requirements(),
                 author_email='dmitry.borovensky@opsys-tech.com',
                 packages=setuptools.find_packages(),
                 zip_safe=False)